<?php
namespace boru\ai\Conversation;

use boru\ai\Schema\Request;

interface ReplayStrategyInterface
{
    /**
     * @param Conversation $conversation
     * @param Request|null $currentRequest
     * @return array
     */
    public function buildReplayMessages(Conversation $conversation, Request $currentRequest = null);
}
