<?php
namespace boru\ai\Embedding;

class NullEmbeddingStore implements EmbeddingStoreInterface
{
    public function findResult($cacheKey)
    {
        return false;
    }

    public function saveResult($cacheKey, EmbeddingRequest $request, EmbeddingResult $result, $ttlSeconds = null)
    {
        return $result;
    }
}
