<?php
namespace boru\ai\Runtime;

interface ToolPromptAdditionInterface
{
    /**
     * Optional instruction fragment to merge into the request when this tool is active.
     *
     * Return an empty string when no prompt addition should be applied.
     *
     * @return string
     */
    public function promptAddition();
}
