<?php
namespace boru\ai\Asset;

class NullAssetStore implements AssetStoreInterface
{
    public function saveAsset(Asset $asset)
    {
        return $asset;
    }

    public function loadAsset($assetId)
    {
        return false;
    }

    public function findAssetBySha256($sha256)
    {
        return false;
    }

    public function saveBinding(ProviderAssetBinding $binding)
    {
        return $binding;
    }

    public function findBinding($assetId, $provider, $purpose = null)
    {
        return false;
    }

    public function listBindings($assetId)
    {
        return array();
    }
}
