<?php
namespace boru\ai\Stream;

interface StreamEventListenerInterface
{
    /**
     * Handle a normalized stream/runtime event.
     *
     * @param StreamEvent $event
     * @return void
     */
    public function handle(StreamEvent $event);
}
