<?php
namespace boru\ai\Mcp;

class NullMcpToolDiscovery implements McpToolDiscoveryInterface
{
    public function discover(McpServerConfig $serverConfig)
    {
        throw new \RuntimeException('No MCP tool discovery configured for server "' . $serverConfig->name() . '"');
    }
}
