<?php
namespace boru\ai\Mcp;

interface McpClientFactoryInterface
{
    /**
     * @param McpServerConfig $serverConfig
     * @return McpClientInterface
     */
    public function makeClient(McpServerConfig $serverConfig);
}
