<?php
namespace boru\ai\Mcp;

interface TcpMcpConnectorInterface
{
    /**
     * @param McpServerConfig $serverConfig
     * @return TcpMcpConnectionInterface
     */
    public function connect(McpServerConfig $serverConfig);
}
