<?php
namespace boru\ai\Runtime;

use boru\ai\Schema\Request;
use boru\ai\Schema\Response;

interface ToolExecutorInterface
{
    /**
     * Resolve any tool calls surfaced in the response and return
     * normalized tool results for the provider to continue with.
     *
     * @param Request $request
     * @param Response $response
     * @return array Array of \boru\ai\Schema\ToolResult
     */
    public function executeToolCalls(Request $request, Response $response);
}
