<?php
namespace boru\ai\Persistence;

use boru\ai\Conversation\Conversation;
use boru\ai\Conversation\ConversationReference;
use boru\ai\Conversation\Turn;

class NullPersistenceProvider implements PersistenceProviderInterface
{
    public function loadConversation(ConversationReference $reference)
    {
        return false;
    }

    public function saveConversation(Conversation $conversation)
    {
        return $conversation;
    }

    public function saveTurn(Conversation $conversation, Turn $turn)
    {
        return $turn;
    }
}
