<?php
namespace boru\ai\Embedding;

interface EmbeddingProviderInterface
{
    /**
     * Generate embeddings for the normalized embedding request.
     *
     * @param EmbeddingRequest $request
     * @return EmbeddingResult
     */
    public function embed(EmbeddingRequest $request);
}
