<?php
namespace boru\ai\Mcp;

class NullMcpToolInvoker implements McpToolInvokerInterface
{
    public function invoke($serverName, $toolName, array $arguments)
    {
        throw new \RuntimeException('No MCP tool invoker configured for server "' . $serverName . '" and tool "' . $toolName . '"');
    }
}
