<?php
$autoload = __DIR__ . '/../vendor/autoload.php';
if (!file_exists($autoload)) {
    throw new RuntimeException('Composer autoload not found. Run composer install first.');
}
require_once $autoload;

\boru\ai\Config\Dotenv::load(array(
    dirname(__DIR__) . '/.env.test.local',
    dirname(__DIR__) . '/.env.test',
    dirname(__DIR__) . '/.env.local',
    dirname(__DIR__) . '/.env',
));
