<?php
namespace boru\ai\Runtime;

class NullToolExecutor implements ToolExecutorInterface
{
    public function executeToolCalls(\boru\ai\Schema\Request $request, \boru\ai\Schema\Response $response)
    {
        return array();
    }
}
