<?php
namespace boru\ai\Mcp;

interface McpToolInvokerInterface
{
    /**
     * @param string $serverName
     * @param string $toolName
     * @param array $arguments
     * @return mixed
     */
    public function invoke($serverName, $toolName, array $arguments);
}
