<?php
namespace boru\ai\Mcp;

interface TcpMcpConnectionInterface
{
    /**
     * @param string $line
     * @return void
     */
    public function writeLine($line);

    /**
     * @return string|false
     */
    public function readLine();

    /**
     * @return void
     */
    public function close();
}
