<?php
namespace boru\ai\Mcp;

interface McpToolDiscoveryInterface
{
    /**
     * @param McpServerConfig $serverConfig
     * @return array
     */
    public function discover(McpServerConfig $serverConfig);
}
