<?php
namespace boru\ai\Mcp;

class NullMcpClientFactory implements McpClientFactoryInterface
{
    public function makeClient(McpServerConfig $serverConfig)
    {
        throw new \RuntimeException('No MCP client factory configured for server "' . $serverConfig->name() . '"');
    }
}
